"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generate = exports.generateStylesheet = exports.styleGetter = exports.camelCase = exports.inject = exports.generateCSS = exports.renderClassName = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }(); /* eslint-disable no-console */
/* global process */

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Strip "@"s and ":"s from a string
 */

var strip = _ramda2.default.replace(/[@:]/g, "");

/**
 * Render class name from array of strings (`null` and `undefined` are ignored)
 */

var renderClassName = exports.renderClassName = _ramda2.default.compose(_ramda2.default.join("--"), _ramda2.default.map(strip), _ramda2.default.reject(_ramda2.default.isNil));

/**
 * Generate CSS for single rule
 */

var rulesReducer = function rulesReducer(name, makeSelector) {
  return function (accum, rule) {
    var key = rule[0];
    var value = rule[1] || key;

    var ruleCSS = "." + makeSelector(name, key) + "{" + name + ":" + value + "}";

    return _ramda2.default.concat(accum, ruleCSS);
  };
};

/**
 * Generate CSS for pseudo class/element
 */

var pseudoReducer = function pseudoReducer(name, rules) {
  return function (accum, pseudo) {

    var pseudoCSS = _ramda2.default.reduce(rulesReducer(name, function (dec, rule) {
      return dec + "--" + rule + "--" + strip(pseudo) + pseudo;
    }), "", rules);

    return _ramda2.default.concat(accum, pseudoCSS);
  };
};

/**
 * Generates CSS for a media query
 */

var mediaQueryReducer = function mediaQueryReducer(name, rules) {
  return function (accum, _ref) {
    var _ref2 = _slicedToArray(_ref, 2);

    var mediaKey = _ref2[0];
    var query = _ref2[1];


    var rulesCSS = _ramda2.default.reduce(rulesReducer(name, function (dec, rule) {
      return dec + "--" + rule + "--" + strip(mediaKey);
    }), "", rules);

    var mediaQueryCSS = "@media " + query + "{" + rulesCSS + "}";

    return _ramda2.default.concat(accum, mediaQueryCSS);
  };
};

/**
 * Generate CSS for a single definition
 */

var definitionReducer = function definitionReducer(accum, _ref3) {
  var name = _ref3.name;
  var rules = _ref3.rules;
  var media = _ref3.media;
  var pseudo = _ref3.pseudo;


  var vanillaRulesCSS = _ramda2.default.reduce(rulesReducer(name, function (dec, rule) {
    return dec + "--" + rule;
  }), "", rules);

  var pseudoCSS = _ramda2.default.reduce(pseudoReducer(name, rules), "", pseudo || []);

  var mediaQueryRulesCSS = _ramda2.default.reduce(mediaQueryReducer(name, rules), "", media || []);

  return "" + accum + vanillaRulesCSS + pseudoCSS + mediaQueryRulesCSS;
};

/**
 * Generate CSS markup
 * @param {object} config
 * @returns {string}
 */

var generateCSS = exports.generateCSS = function generateCSS(defs) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  var css = _ramda2.default.reduce(definitionReducer, "", defs);

  return "" + (options.globals || "") + css;
};

/**
 * Inject CSS into <head>
 * @param {string}
 * @returns {void}
 */

var inject = exports.inject = function inject(css) {
  var style = document.createElement("style");
  style.type = "text/css";
  style.appendChild(document.createTextNode(css));

  document.head.appendChild(style);
};

/**
 * Convert kebab-case to camelCase
 */

var camelCase = exports.camelCase = function camelCase(string) {
  return _ramda2.default.replace(/-([a-z])/g, function (_, match) {
    return _ramda2.default.toUpper(match);
  }, string);
};

/**
 * Retrieve classname from config
 */

var styleGetter = exports.styleGetter = function styleGetter(_ref4) {
  var name = _ref4.name;
  var rules = _ref4.rules;
  var media = _ref4.media;
  var pseudo = _ref4.pseudo;
  return function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var rule = args[0];
    var arg = args[1];

    // This is a convenience for being able to call the getter function with
    // arguments but without generating no output.
    // This is useful when passing props directly

    if (rule === null) return null;

    if (process.env.NODE_ENV !== "production") {
      if (!_ramda2.default.find(_ramda2.default.propEq(0, rule), rules)) {
        return console.error("Couldn't find '" + rule + "' rule for '" + name + "' definition. Available rules are: " + rules.map(function (_ref5) {
          var _ref6 = _slicedToArray(_ref5, 1);

          var ruleName = _ref6[0];
          return ruleName;
        }).join(", "));
      }

      if (arg) {
        var isPseudo = arg.startsWith(":");
        var isMedia = arg.startsWith("@");

        if (isMedia) {
          if (!media) {
            return console.error("You're trying to access '" + media + "' media query for '" + name + "' definition, but the definition doesn't include any 'media' property at all. Try checking your funcss definitions for errors.");
          }
          if (!_ramda2.default.find(_ramda2.default.propEq(0, arg), media)) {
            return console.error("Couldn't find '" + media + "' media query for '" + name + "' definition. Available media queries are: " + media.map(function (_ref7) {
              var _ref8 = _slicedToArray(_ref7, 1);

              var mediaName = _ref8[0];
              return mediaName;
            }).join(", "));
          }
        }

        if (isPseudo) {
          if (!pseudo) {
            return console.error("You're trying to access '" + pseudo + "' pseudo class/element for '" + name + "' definition, but the definition doesn't include any 'pseudo' property at all. Try checking your funcss definitions for errors.");
          }
          if (!_ramda2.default.find(_ramda2.default.equals(arg), pseudo)) {
            return console.error("Couldn't find '" + pseudo + "' psedo class/element for '" + name + "' definition. Available media queries are: " + pseudo.join(", "));
          }
        }
      }
    }

    return renderClassName([name].concat(args));
  };
};

/**
 * Create stylesheet object
 */

var generateStylesheet = exports.generateStylesheet = function generateStylesheet(defs) {
  return _ramda2.default.reduce(function (acc, def) {
    return _ramda2.default.assoc(camelCase(def.name), styleGetter(def), acc);
  }, {}, defs);
};

/**
 * Generate CSS string and stylesheet object
 */

var generate = exports.generate = function generate(config, options) {
  return {
    css: generateCSS(config, options),
    stylesheet: generateStylesheet(config)
  };
};

/**
 * Main
 */

exports.default = function (config, options) {
  var _generate = generate(config, options);

  var css = _generate.css;
  var stylesheet = _generate.stylesheet;

  inject(css);
  return stylesheet;
};